// const SYMBOL_REGEXP = /^Symbol\((.*)\)(.*)$/;
// const NEWLINE_REGEXP = /\n/gi;
function _formatArrayItems(v, config, context) {
    if (!v.length)
        return '';
    const beforeIndent = context.currentIndent;
    const currentIndent = context.currentIndent += config.indent;
    const beforeIndentStr = ' '.repeat(beforeIndent);
    const currentIndentStr = ' '.repeat(currentIndent);
    const ret = Array.from(v).map((val, idx) => currentIndentStr + (idx in v ?
        context.format(val, config, context)
        : '')).join(config.inline ? ',' : `,${config.innerSeparator}`);
    context.currentIndent -= config.indent;
    return `\n${ret}\n${beforeIndentStr}`;
}
function _formatObjectEntries(entries, config, context, separator) {
    const beforeIndent = context.currentIndent;
    const beforeIndentStr = ' '.repeat(beforeIndent);
    const currentIndent = context.currentIndent += config.indent;
    const currentIndentStr = ' '.repeat(currentIndent);
    const ret = entries
        .map(([key, value]) => `${currentIndentStr}${context.format(key, config, context)}${separator}${context.format(value, config, context)}`)
        .join(',' + config.innerSeparator);
    context.currentIndent -= config.indent;
    return `\n${ret}\n${beforeIndentStr}`;
}
export function formatNumber(v) {
    return Object.is(v, -0) ? '-0' : String(v);
}
export function formatString(v) {
    return `"${v}"`;
}
export function formatBigInt(v) {
    return String(`${v}n`);
}
export function formatFunction(v, options) {
    if (options?.ignoreFunctionName) {
        return '[Function]';
    }
    return `[Function ${v.name || 'anonymous'}]`;
}
export function foramtSymbol(v) {
    return v.toString();
}
export function formatError(v) {
    return `[${Error.prototype.toString.call(v)}]`;
}
export function formatRegExp(v) {
    return RegExp.prototype.toString.call(v);
}
export function formatDate(v) {
    return isNaN(+v) ? 'Date { NaN }' : Date.prototype.toISOString.call(v);
}
export function formatArray(v, config, context) {
    return `Array [${_formatArrayItems(v, config, context)}]`;
}
export function formatArguments(v, config, context) {
    return `Arguments [${_formatArrayItems(v, config, context)}]`;
}
export function formatTypedArray(v, config, context) {
    return `${v.constructor.name} [${_formatArrayItems(v, config, context)}]`;
}
export function formatObject(v, config, context) {
    const keys = [
        ...Object.keys(v).sort(),
        ...Object.getOwnPropertySymbols(v).filter(symbol => (Object.getOwnPropertyDescriptor(v, symbol).enumerable))
    ].map(key => [key, v[key]]);
    return `${v.constructor?.name ?? 'Object'} {${_formatObjectEntries(keys, config, context, ': ')}}`;
}
export function formatMap(v, config, context) {
    if (v.size === 0) {
        return 'Map {}';
    }
    return `Map {${_formatObjectEntries(Array.from(v), config, context, ' => ')}}`;
}
export function formatSet(v, config, context) {
    if (v.size === 0) {
        return 'Set {}';
    }
    return `Set {${_formatArrayItems(Array.from(v), config, context)}}`;
}
//# sourceMappingURL=format.js.map