import { objectToString } from './proto.mjs';
export function isUndefined(v) {
    return v === undefined;
}
export function isNull(v) {
    return v === null;
}
export function isBoolean(v) {
    return v === true || v === false;
}
export function isNumber(v) {
    return typeof v === 'number';
}
export function isBigint(v) {
    return typeof v === 'bigint';
}
export function isString(v) {
    return typeof v === 'string';
}
export function isFunction(v) {
    const tag = objectToString(v);
    return typeof v === 'function'
        || tag === '[object Function]'
        || tag === '[object GeneratorFunction]'
        || tag === '[object AsyncFunction]';
}
export function isSymbol(v) {
    return typeof v === 'symbol' || objectToString(v) === '[object Symbol]';
}
export function isWeakMap(v) {
    return objectToString(v) === '[object WeakMap]';
}
export function isWeakSet(v) {
    return objectToString(v) === '[object WeakSet]';
}
export function isRegExp(v) {
    return objectToString(v) === '[object RegExp]';
}
export function isDate(v) {
    return objectToString(v) === '[object Date]';
}
export function isError(v) {
    return objectToString(v) === '[object Error]' || v instanceof Error;
}
export function isArguments(v) {
    return objectToString(v) === '[object Arguments]';
}
export function isObject(v) {
    return v !== null && !Array.isArray(v) && typeof v === 'object';
}
export function isMap(v) {
    return objectToString(v) === '[object Map]';
}
export function isSet(v) {
    return objectToString(v) === '[object Set]';
}
export function isArray(v) {
    return Array.isArray(v);
}
export function isTypedArray(v) {
    const tag = objectToString(v);
    if (tag === '[object ArrayBuffer]'
        || tag === '[object DataView]'
        || tag === '[object Int8Array]'
        || tag === '[object Int16Array]'
        || tag === '[object Int32Array]'
        || tag === '[object Uint8Array]'
        || tag === '[object Uint8ClampedArray]'
        || tag === '[object Uint16Array]'
        || tag === '[object Uint32Array]'
        || tag === '[object Float32Array]'
        || tag === '[object Float64Array]') {
        return true;
    }
    return false;
}
//# sourceMappingURL=is.js.map